

#include <EEPROM.h>

#define MAX_INPUT 3
#define MAX_PARAMETER 46
#define MAX_ATTENUATION -472

int16_t parameters[MAX_PARAMETER+1] ;


void writeIntIntoEEPROM(int address, int16_t number)
{ 
  byte byte1 = number >> 8;
  byte byte2 = number & 0xFF;

  EEPROM.write(address, byte1);
  EEPROM.write(address + 1, byte2);
}

int readIntFromEEPROM(int address)
{
  byte byte1 = EEPROM.read(address);
  byte byte2 = EEPROM.read(address + 1);

  return (byte1 << 8) + byte2;
}

void initialize_parameters()
{
for (int i=0; i <MAX_PARAMETER;i++)
    parameters[i]=0;
    
    parameters[0] = 0 ;
    parameters[10] = -472 ;
    parameters[11] = -50 ;
    parameters[12] = -472 ;
    parameters[13] = -50 ;

    parameters[20] = -20 ;
    parameters[21] = -20 ;
    parameters[22] = -20 ;
    parameters[23] = -20 ;

    parameters[30] = -43 ;
    parameters[31] = -43 ;
    parameters[32] = -43 ;
    parameters[33] = -43 ;

    parameters[40] = -17 ;
    parameters[41] = -17 ;
    parameters[42] = -16 ;
    parameters[43] = -16 ;
    

}
// Routines for reading and writing from EEPROM

void writeParameters(){
   #ifdef DEBUG
     Serial.println("WRITE");
   #endif
  int eeAddress = 0;
  for(int i=0;i<MAX_PARAMETER;i++) {
    if(readIntFromEEPROM(i)!=abs(parameters[i])) // Check an see if there are any changes
    {   
      #ifdef DEBUG
        Serial.print(i);
        Serial.print(" ");
      #endif
        writeIntIntoEEPROM(eeAddress,abs(parameters[i]));
        #ifdef DEBUG
        Serial.print(parameters[i]);
        Serial.print(" ");
        Serial.println(EEPROM.read(eeAddress));
        #endif
        
    }
    eeAddress += sizeof(int16_t);
  } 
}


void readParameters(){
   #ifdef DEBUG 
    Serial.println("READ");
   #endif
  int eeAddress = 0;
  for (int i=0;i<MAX_PARAMETER;i++) 
  {
      #ifdef DEBUG
          Serial.print(i);
          Serial.print(" ");
          Serial.print( readIntFromEEPROM(eeAddress));
      #endif
      parameters[i] = (readIntFromEEPROM(eeAddress) * -1 );
      
      #ifdef DEBUG
          Serial.print(" ");
          Serial.println(parameters[i]);
      #endif
      
      eeAddress += sizeof(int16_t);
    }
     
}


void eraseEEPROM()
{
for (int i = 0 ; i < EEPROM.length() ; i++) {
    EEPROM.write(i, 0);
  }

}

void setup() {
    
  eraseEEPROM() ;
  initialize_parameters();
  writeParameters() ;
  

}

void loop() {
  // put your main code here, to run repeatedly:

}
